//!
//!

use super::super::super::compositions;
use super::super::super::repositories;
use beteran_common_rust as bcr;
use beteran_protobuf_rust as bpr;
use diesel::{
  r2d2::{ConnectionManager, Pool},
  PgConnection,
};
use prost::Message;
use std::str::FromStr;

///
pub struct Service<'a> {
  connection_broker: nats::asynk::Connection,
  queue_broker: String,
  pool: Pool<ConnectionManager<PgConnection>>,
  site_composition: compositions::site::composition::Composition,
  member_composition: compositions::member::composition::Composition,
  member_repository: repositories::member::repository::Repository,
  member_session_repository: repositories::member_session::repository::Repository,
  member_bank_account_repository: repositories::member_bank_account::repository::Repository,
  member_game_setting_repository: repositories::member_game_setting::repository::Repository,
  member_settlement_setting_repository:
    repositories::member_settlement_setting::repository::Repository,
  argon2_config: argon2::Config<'a>,
  password_salt: String,
}

impl std::fmt::Debug for Service<'_> {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Service of service.member.service.identity")
      .finish()
  }
}

impl Service<'_> {
  ///
  pub fn new(
    connection_broker: nats::asynk::Connection,
    queue_broker: String,
    pool: Pool<ConnectionManager<PgConnection>>,
    password_salt: String,
  ) -> Service<'static> {
    Service {
      connection_broker,
      queue_broker,
      pool,
      site_composition: compositions::site::composition::Composition::new(),
      member_composition: compositions::member::composition::Composition::new(),
      member_repository: repositories::member::repository::Repository::new(),
      member_session_repository: repositories::member_session::repository::Repository::new(),
      member_bank_account_repository:
        repositories::member_bank_account::repository::Repository::new(),
      member_game_setting_repository:
        repositories::member_game_setting::repository::Repository::new(),
      member_settlement_setting_repository:
        repositories::member_settlement_setting::repository::Repository::new(),
      argon2_config: argon2::Config::default(),
      password_salt,
    }
  }

  pub async fn subscribe(&self) -> std::result::Result<(), std::boxed::Box<dyn std::error::Error>> {
    futures::try_join!(
      self.create_member(),
      self.list_members(),
      self.get_member(),
      self.get_member_by_username(),
      self.get_member_b(),
      self.get_member_b_by_username(),
      self.update_member(),
      self.update_member_for_password(),
      self.update_member_for_state(),
      self.delete_member(),
    )
    .map(|_| ())
  }

  fn check_site(
    &self,
    conn: &diesel::PgConnection,
    url: Option<String>,
    site_id: uuid::Uuid,
  ) -> Result<repositories::site::models::Site, bcr::error::rpc::Error> {
    match self
      .site_composition
      .select_by_url(conn, url, site_id)
      .map_err(|e| {
        bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
          code: bpr::protobuf::rpc::Error::SERVER_00,
          message: format!("server {}", e),
          data: None,
        })
      })? {
      Some(s) => Ok(s),
      None => Err(bcr::error::rpc::Error::InvalidParams(
        bcr::error::rpc::InvalidParams {
          message: "invalid site_url information".to_string(),
          detail: bcr::error::rpc::InvalidParamsDetail {
            location: "request".to_string(),
            param: "client.site_url".to_string(),
            value: "".to_string(),
            error_type: bcr::error::rpc::InvalidParamsType::None,
            message: "".to_string(),
          },
        },
      )),
    }
  }

  async fn create_member(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::ss::member::SUBJECT_CREATE_MEMBER,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let req =
          bpr::ss::member::CreateMemberRequest::decode(message.data.as_slice()).map_err(|e| {
            bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
              message: format!("invalid request: {}", e),
            })
          })?;

        let client = match req.client {
          Some(c) => c,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid client information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "client".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };
        let request = match req.request {
          Some(r) => r,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid request information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "request".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };
        let site_id = uuid::Uuid::from_str(request.site_id.as_str()).map_err(|e| {
          bcr::error::rpc::Error::InvalidParams(bcr::error::rpc::InvalidParams {
            message: "invalid site_id param".to_string(),
            detail: bcr::error::rpc::InvalidParamsDetail {
              location: "request".to_string(),
              param: "site_id".to_string(),
              value: request.site_id.clone(),
              error_type: bcr::error::rpc::InvalidParamsType::Required,
              message: e.to_string(),
            },
          })
        })?;
        let member_class_id =
          uuid::Uuid::from_str(request.member_class_id.as_str()).map_err(|e| {
            bcr::error::rpc::Error::InvalidParams(bcr::error::rpc::InvalidParams {
              message: "invalid member_class_id param".to_string(),
              detail: bcr::error::rpc::InvalidParamsDetail {
                location: "request".to_string(),
                param: "member_class_id".to_string(),
                value: request.member_class_id.clone(),
                error_type: bcr::error::rpc::InvalidParamsType::Required,
                message: e.to_string(),
              },
            })
          })?;
        let member_level_id =
          uuid::Uuid::from_str(request.member_level_id.as_str()).map_err(|e| {
            bcr::error::rpc::Error::InvalidParams(bcr::error::rpc::InvalidParams {
              message: "invalid member_level_id param".to_string(),
              detail: bcr::error::rpc::InvalidParamsDetail {
                location: "request".to_string(),
                param: "member_level_id".to_string(),
                value: request.member_level_id.clone(),
                error_type: bcr::error::rpc::InvalidParamsType::Required,
                message: e.to_string(),
              },
            })
          })?;

        let conn = self.pool.get().map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        let referrer_member_username = match &request.referrer_member_username {
          Some(d) => {
            if d.trim().is_empty() {
              None
            } else {
              Some(d.clone())
            }
          },
          None => None,
        };

        let parent_member_id = match referrer_member_username {
          Some(referrer_member_username) => {
            match self
              .member_repository
              .select_by_username(&conn, referrer_member_username.as_str())
              .map_err(|e| {
                bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                  code: bpr::protobuf::rpc::Error::SERVER_00,
                  message: format!("server {}", e),
                  data: None,
                })
              })? {
              Some(m) => Some(m.id),
              None => {
                return Err(bcr::error::rpc::Error::InvalidParams(
                  bcr::error::rpc::InvalidParams {
                    message: "invalid referrer_member_username information".to_string(),
                    detail: bcr::error::rpc::InvalidParamsDetail {
                      location: "request".to_string(),
                      param: "referrer_member_username".to_string(),
                      value: referrer_member_username.to_string(),
                      error_type: bcr::error::rpc::InvalidParamsType::None,
                      message: "not exist".to_string(),
                    },
                  },
                ));
              }
            }
          }
          None => None,
        };

        match self
          .member_repository
          .select_by_username(&conn, &request.username)
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })? {
          Some(m) => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid username".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "username".to_string(),
                  value: request.username.to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::None,
                  message: "duplicated".to_string(),
                },
              },
            ));
          }
          None => {}
        };

        match self
          .member_repository
          .select_by_nickname(&conn, &request.nickname)
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })? {
          Some(m) => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid nickname".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "nickname".to_string(),
                  value: request.nickname.to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::None,
                  message: "duplicated".to_string(),
                },
              },
            ));
          }
          None => {}
        };

        let password_hash = argon2::hash_encoded(
          request.password.as_bytes(),
          self.password_salt.as_bytes(),
          &self.argon2_config,
        )
        .map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        let new_member = repositories::member::models::NewMember {
          site_id,
          member_class_id,
          member_level_id,
          parent_member_id,
          username: request.username,
          password: password_hash,
          nickname: request.nickname,
          state: repositories::member::schema::MemberState::from(request.state),
          mobile_phone_number: request.mobile_phone_number,
        };

        let m = self
          .member_repository
          .insert(&conn, &new_member)
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        if let Some(a) = request.bank_account {
          let bank_id = uuid::Uuid::from_str(a.bank_id.as_str()).map_err(|e| {
            bcr::error::rpc::Error::InvalidParams(bcr::error::rpc::InvalidParams {
              message: "invalid bank_account.bank_id param".to_string(),
              detail: bcr::error::rpc::InvalidParamsDetail {
                location: "request".to_string(),
                param: "bank_account.bank_id".to_string(),
                value: a.bank_id.clone(),
                error_type: bcr::error::rpc::InvalidParamsType::Required,
                message: e.to_string(),
              },
            })
          })?;

          if a.exchange_password.trim().eq("") {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid bank_account.exchange_password param".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "bank_account.exchange_password".to_string(),
                  value: a.exchange_password,
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "bank_account.exchange_password is required".to_string(),
                },
              },
            ));
          }

          let exchange_password_hash = argon2::hash_encoded(
            a.exchange_password.as_bytes(),
            self.password_salt.as_bytes(),
            &self.argon2_config,
          )
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

          let new_member_bank_account =
            repositories::member_bank_account::models::NewMemberBankAccount {
              member_id: m.id,
              bank_id,
              name: a.name,
              account_number: a.account_number,
              exchange_password: exchange_password_hash,
              memo: None,
            };
          self
            .member_bank_account_repository
            .insert(&conn, &new_member_bank_account)
            .map_err(|e| {
              bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                code: bpr::protobuf::rpc::Error::SERVER_00,
                message: format!("server {}", e),
                data: None,
              })
            })?;
        }

        match request.game_setting {
          Some(a) => {
            let new_member_game_setting =
              repositories::member_game_setting::models::NewMemberGameSetting {
                member_id: m.id,
                can_bet_casino: a.can_bet_casino,
                can_bet_slot: a.can_bet_slot,
                can_bet_powerball: a.can_bet_powerball,
                can_bet_powerladder: a.can_bet_powerladder,
                can_bet_eos: a.can_bet_eos,
                can_bet_bogglepowerball: a.can_bet_bogglepowerball,
              };
            self
              .member_game_setting_repository
              .insert(&conn, &new_member_game_setting)
              .map_err(|e| {
                bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                  code: bpr::protobuf::rpc::Error::SERVER_00,
                  message: format!("server {}", e),
                  data: None,
                })
              })?;
          }
          None => {
            let new_member_game_setting =
              repositories::member_game_setting::models::NewMemberGameSettingForDefault {
                member_id: m.id,
              };
            self
              .member_game_setting_repository
              .insert_with_default(&conn, &new_member_game_setting)
              .map_err(|e| {
                bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                  code: bpr::protobuf::rpc::Error::SERVER_00,
                  message: format!("server {}", e),
                  data: None,
                })
              })?;
          }
        }

        match request.settlement_setting {
          Some(a) => {
            let new_member_settlement_setting =
              repositories::member_settlement_setting::models::NewMemberSettlementSetting {
                member_id: m.id,
                can_exchange: a.can_exchange,
                can_first_charge_comp: a.can_first_charge_comp,
                can_per_charge_comp: a.can_per_charge_comp,
                manual_payment_type_for_partner:
                  repositories::member_settlement_setting::schema::ManualPaymentType::from(
                    a.manual_payment_type_for_partner,
                  ),
                settlement_type:
                  repositories::member_settlement_setting::schema::SettlementType::from(
                    a.settlement_type,
                  ),
                rate_casino: a.rate_casino,
                rate_casino_loosing: a.rate_casino_loosing,
                rate_casino_bacara: a.rate_casino_bacara,
                rate_casino_roulette: a.rate_casino_roulette,
                rate_casino_dragon_tiger: a.rate_casino_dragon_tiger,
                rate_slot: a.rate_slot,
                rate_slot_loosing: a.rate_slot_loosing,
                rate_powerball_single: a.rate_powerball_single,
                rate_powerball_combo: a.rate_powerball_combo,
                rate_powerladder_single: a.rate_powerladder_single,
                rate_powerladder_combo: a.rate_powerladder_combo,
                rate_eos_single: a.rate_eos_single,
                rate_eos_combo: a.rate_eos_combo,
                rate_bogglepowerball_single: a.rate_bogglepowerball_single,
                rate_bogglepowerball_combo: a.rate_bogglepowerball_combo,
              };
            self
              .member_settlement_setting_repository
              .insert(&conn, &new_member_settlement_setting)
              .map_err(|e| {
                bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                  code: bpr::protobuf::rpc::Error::SERVER_00,
                  message: format!("server {}", e),
                  data: None,
                })
              })?;
          }
          None => {
            let new_member_settlement_setting =
              repositories::member_settlement_setting::models::NewMemberSettlementSettingForDefault {
                member_id: m.id,
              };
            self
              .member_settlement_setting_repository
              .insert_with_default(&conn, &new_member_settlement_setting)
              .map_err(|e| {
                bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                  code: bpr::protobuf::rpc::Error::SERVER_00,
                  message: format!("server {}", e),
                  data: None,
                })
              })?;
          }
        }

        let member = self.member_composition.select(&conn, m.id).map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        message
          .respond(
            bpr::ss::member::CreateMemberResponse {
              error: None,
              result: Some(bpr::ss::member::create_member_response::Result {
                member: member.as_ref().map(bpr::models::member::MemberModel::from),
              }),
            }
            .encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        self
          .connection_broker
          .publish(
            bpr::ss::member::EVENT_SUBJECT_AFTER_CREATE_MEMBER,
            bpr::ss::member::AfterCreateMemberEvent {
              client: Some(client),
              event: Some(bpr::ss::member::after_create_member_event::Event {
                member: member.as_ref().map(bpr::models::member::MemberModel::from),
              }),
            }
            .encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::ss::member::CreateMemberResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn list_members(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::ss::member::SUBJECT_LIST_MEMBERS,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let req =
          bpr::ss::member::ListMembersRequest::decode(message.data.as_slice()).map_err(|e| {
            bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
              message: format!("invalid request: {}", e),
            })
          })?;

        let client = match req.client {
          Some(c) => c,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid client information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "client".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };
        let request = match req.request {
          Some(r) => r,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid request information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "request".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };

        let search = match request.search {
          Some(s) => {
            let site_id = match s.site_id {
              Some(d) => match uuid::Uuid::from_str(d.as_str()) {
                Ok(dd) => Some(dd),
                Err(e) => {
                  return Err(bcr::error::rpc::Error::InvalidParams(
                    bcr::error::rpc::InvalidParams {
                      message: "invalid site_id param".to_string(),
                      detail: bcr::error::rpc::InvalidParamsDetail {
                        location: "request".to_string(),
                        param: "site_id".to_string(),
                        value: d.clone(),
                        error_type: bcr::error::rpc::InvalidParamsType::Required,
                        message: e.to_string(),
                      },
                    },
                  ));
                }
              },
              None => None,
            };
            let member_class_id = match s.member_class_id {
              Some(d) => match uuid::Uuid::from_str(d.as_str()) {
                Ok(dd) => Some(dd),
                Err(e) => {
                  return Err(bcr::error::rpc::Error::InvalidParams(
                    bcr::error::rpc::InvalidParams {
                      message: "invalid member_class_id param".to_string(),
                      detail: bcr::error::rpc::InvalidParamsDetail {
                        location: "request".to_string(),
                        param: "member_class_id".to_string(),
                        value: d.clone(),
                        error_type: bcr::error::rpc::InvalidParamsType::Required,
                        message: e.to_string(),
                      },
                    },
                  ));
                }
              },
              None => None,
            };
            let member_level_id = match s.member_level_id {
              Some(d) => match uuid::Uuid::from_str(d.as_str()) {
                Ok(dd) => Some(dd),
                Err(e) => {
                  return Err(bcr::error::rpc::Error::InvalidParams(
                    bcr::error::rpc::InvalidParams {
                      message: "invalid member_level_id param".to_string(),
                      detail: bcr::error::rpc::InvalidParamsDetail {
                        location: "request".to_string(),
                        param: "member_level_id".to_string(),
                        value: d.clone(),
                        error_type: bcr::error::rpc::InvalidParamsType::Required,
                        message: e.to_string(),
                      },
                    },
                  ));
                }
              },
              None => None,
            };
            let parent_member_id = match s.parent_member_id {
              Some(d) => match uuid::Uuid::from_str(d.as_str()) {
                Ok(dd) => Some(dd),
                Err(e) => {
                  return Err(bcr::error::rpc::Error::InvalidParams(
                    bcr::error::rpc::InvalidParams {
                      message: "invalid parent_member_id param".to_string(),
                      detail: bcr::error::rpc::InvalidParamsDetail {
                        location: "request".to_string(),
                        param: "parent_member_id".to_string(),
                        value: d.clone(),
                        error_type: bcr::error::rpc::InvalidParamsType::Required,
                        message: e.to_string(),
                      },
                    },
                  ));
                }
              },
              None => None,
            };

            Some(repositories::member::models::FindAllSearch {
              site_id,
              member_class_id,
              member_level_id,
              parent_member_id,
              username: s.username,
              usernames: s.usernames,
              username_like: s.username_like,
              nickname: s.nickname,
              nicknames: s.nicknames,
              nickname_like: s.nickname_like,
              mobile_phone_number_like: s.mobile_phone_number_like,
              last_signined_ip: s.last_signined_ip,
              state: s.state.map(repositories::member::schema::MemberState::from),
              deleted_at: s.deleted_at.map(|d| d > 0),
            })
          }
          None => None,
        };

        let conn = self.pool.get().map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        let find_all = repositories::member::models::FindAll {
          pagination: request
            .pagination
            .as_ref()
            .map(bcr::pagination::Pagination::from),
          sorts: Some(
            request
              .sorts
              .iter()
              .map(bcr::pagination::Sort::from)
              .collect(),
          ),
          search,
        };

        let total_count = self
          .member_composition
          .select_all_count(&conn, &find_all)
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        let member_list = self
          .member_composition
          .select_all(&conn, &find_all)
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        message
          .respond(
            bpr::ss::member::ListMembersResponse {
              error: None,
              result: Some(bpr::ss::member::list_members_response::Result {
                total_count: total_count as u64,
                members: member_list
                  .iter()
                  .map(bpr::models::member::MemberModel::from)
                  .collect(),
              }),
            }
            .encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::ss::member::ListMembersResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn get_member(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::ss::member::SUBJECT_GET_MEMBER,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let req =
          bpr::ss::member::GetMemberRequest::decode(message.data.as_slice()).map_err(|e| {
            bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
              message: format!("invalid request: {}", e),
            })
          })?;

        let client = match req.client {
          Some(c) => c,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid client information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "client".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };
        let request = match req.request {
          Some(r) => r,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid request information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "request".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };
        let id = uuid::Uuid::from_str(request.id.as_str()).map_err(|e| {
          bcr::error::rpc::Error::InvalidParams(bcr::error::rpc::InvalidParams {
            message: "invalid id param".to_string(),
            detail: bcr::error::rpc::InvalidParamsDetail {
              location: "request".to_string(),
              param: "id".to_string(),
              value: request.id.clone(),
              error_type: bcr::error::rpc::InvalidParamsType::Required,
              message: e.to_string(),
            },
          })
        })?;

        let conn = self.pool.get().map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        let m = self.member_composition.select(&conn, id).map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        message
          .respond(
            bpr::ss::member::GetMemberResponse {
              error: None,
              result: Some(bpr::ss::member::get_member_response::Result {
                member: m.map(|d| bpr::models::member::MemberModel::from(&d)),
              }),
            }
            .encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::ss::member::GetMemberResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn get_member_by_username(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::ss::member::SUBJECT_GET_MEMBER_BY_USERNAME,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let req = bpr::ss::member::GetMemberByUsernameRequest::decode(message.data.as_slice())
          .map_err(|e| {
            bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
              message: format!("invalid request: {}", e),
            })
          })?;

        let client = match req.client {
          Some(c) => c,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid client information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "client".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };
        let request = match req.request {
          Some(r) => r,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid request information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "request".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };
        let conn = self.pool.get().map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        let m = self
          .member_composition
          .select_by_username(&conn, request.username.as_str())
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        message
          .respond(
            bpr::ss::member::GetMemberByUsernameResponse {
              error: None,
              result: Some(bpr::ss::member::get_member_by_username_response::Result {
                member: m.map(|d| bpr::models::member::MemberModel::from(&d)),
              }),
            }
            .encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::ss::member::GetMemberByUsernameResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn get_member_b(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::ss::member::SUBJECT_GET_MEMBER_B,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let req =
          bpr::ss::member::GetMemberBRequest::decode(message.data.as_slice()).map_err(|e| {
            bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
              message: format!("invalid request: {}", e),
            })
          })?;

        let client = match req.client {
          Some(c) => c,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid client information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "client".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };
        let request = match req.request {
          Some(r) => r,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid request information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "request".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };
        let id = uuid::Uuid::from_str(request.id.as_str()).map_err(|e| {
          bcr::error::rpc::Error::InvalidParams(bcr::error::rpc::InvalidParams {
            message: "invalid id param".to_string(),
            detail: bcr::error::rpc::InvalidParamsDetail {
              location: "request".to_string(),
              param: "id".to_string(),
              value: request.id.clone(),
              error_type: bcr::error::rpc::InvalidParamsType::Required,
              message: e.to_string(),
            },
          })
        })?;

        let conn = self.pool.get().map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        let m = self.member_repository.select(&conn, id).map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        message
          .respond(
            bpr::ss::member::GetMemberBResponse {
              error: None,
              result: Some(bpr::ss::member::get_member_b_response::Result {
                member: m.map(|d| bpr::models::member::Member::from(&d)),
              }),
            }
            .encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::ss::member::GetMemberBResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn get_member_b_by_username(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::ss::member::SUBJECT_GET_MEMBER_B_BY_USERNAME,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let req = bpr::ss::member::GetMemberBByUsernameRequest::decode(message.data.as_slice())
          .map_err(|e| {
            bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
              message: format!("invalid request: {}", e),
            })
          })?;

        let client = match req.client {
          Some(c) => c,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid client information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "client".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };
        let request = match req.request {
          Some(r) => r,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid request information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "request".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };
        let conn = self.pool.get().map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        let m = self
          .member_repository
          .select_by_username(&conn, request.username.as_str())
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        message
          .respond(
            bpr::ss::member::GetMemberBByUsernameResponse {
              error: None,
              result: Some(bpr::ss::member::get_member_b_by_username_response::Result {
                member: m.map(|d| bpr::models::member::Member::from(&d)),
              }),
            }
            .encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::ss::member::GetMemberBByUsernameResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn update_member(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::ss::member::SUBJECT_UPDATE_MEMBER,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let req =
          bpr::ss::member::UpdateMemberRequest::decode(message.data.as_slice()).map_err(|e| {
            bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
              message: format!("invalid request: {}", e),
            })
          })?;

        let client = match req.client {
          Some(c) => c,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid client information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "client".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };
        let request = match req.request {
          Some(r) => r,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid request information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "request".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };

        let id = uuid::Uuid::from_str(request.id.as_str()).map_err(|e| {
          bcr::error::rpc::Error::InvalidParams(bcr::error::rpc::InvalidParams {
            message: "invalid id param".to_string(),
            detail: bcr::error::rpc::InvalidParamsDetail {
              location: "request".to_string(),
              param: "id".to_string(),
              value: request.id.clone(),
              error_type: bcr::error::rpc::InvalidParamsType::Required,
              message: e.to_string(),
            },
          })
        })?;

        let site_id = match request.site_id {
          Some(d) => match uuid::Uuid::from_str(d.as_str()) {
            Ok(dd) => Some(dd),
            Err(e) => {
              return Err(bcr::error::rpc::Error::InvalidParams(
                bcr::error::rpc::InvalidParams {
                  message: "invalid site_id param".to_string(),
                  detail: bcr::error::rpc::InvalidParamsDetail {
                    location: "request".to_string(),
                    param: "site_id".to_string(),
                    value: d.clone(),
                    error_type: bcr::error::rpc::InvalidParamsType::Required,
                    message: e.to_string(),
                  },
                },
              ));
            }
          },
          None => None,
        };

        let member_level_id = match request.member_level_id {
          Some(d) => match uuid::Uuid::from_str(d.as_str()) {
            Ok(dd) => Some(dd),
            Err(e) => {
              return Err(bcr::error::rpc::Error::InvalidParams(
                bcr::error::rpc::InvalidParams {
                  message: "invalid member_level_id param".to_string(),
                  detail: bcr::error::rpc::InvalidParamsDetail {
                    location: "request".to_string(),
                    param: "member_level_id".to_string(),
                    value: d.clone(),
                    error_type: bcr::error::rpc::InvalidParamsType::Required,
                    message: e.to_string(),
                  },
                },
              ));
            }
          },
          None => None,
        };

        let conn = self.pool.get().map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        let modify_member = repositories::member::models::ModifyMember {
          site_id,
          member_level_id,
          mobile_phone_number: request.mobile_phone_number,
        };

        let _affected = self
          .member_repository
          .update(&conn, id, &modify_member)
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        if let Some(a) = request.bank_account {
          let id = uuid::Uuid::from_str(a.id.as_str()).map_err(|e| {
            bcr::error::rpc::Error::InvalidParams(bcr::error::rpc::InvalidParams {
              message: "invalid bank_account.id param".to_string(),
              detail: bcr::error::rpc::InvalidParamsDetail {
                location: "request".to_string(),
                param: "bank_account.id".to_string(),
                value: a.id.clone(),
                error_type: bcr::error::rpc::InvalidParamsType::Required,
                message: e.to_string(),
              },
            })
          })?;

          let bank_id = uuid::Uuid::from_str(a.bank_id.as_str()).map_err(|e| {
            bcr::error::rpc::Error::InvalidParams(bcr::error::rpc::InvalidParams {
              message: "invalid bank_account.bank_id param".to_string(),
              detail: bcr::error::rpc::InvalidParamsDetail {
                location: "request".to_string(),
                param: "bank_account.bank_id".to_string(),
                value: a.bank_id.clone(),
                error_type: bcr::error::rpc::InvalidParamsType::Required,
                message: e.to_string(),
              },
            })
          })?;

          let modify_member_bank_account =
            repositories::member_bank_account::models::ModifyMemberBankAccount {
              bank_id,
              name: a.name,
              account_number: a.account_number,
              memo: None,
            };
          self
            .member_bank_account_repository
            .update(&conn, id, &modify_member_bank_account)
            .map_err(|e| {
              bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                code: bpr::protobuf::rpc::Error::SERVER_00,
                message: format!("server {}", e),
                data: None,
              })
            })?;
        }

        if let Some(a) = request.game_setting {
          let id = uuid::Uuid::from_str(a.id.as_str()).map_err(|e| {
            bcr::error::rpc::Error::InvalidParams(bcr::error::rpc::InvalidParams {
              message: "invalid game_setting.id param".to_string(),
              detail: bcr::error::rpc::InvalidParamsDetail {
                location: "request".to_string(),
                param: "game_setting.id".to_string(),
                value: a.id.clone(),
                error_type: bcr::error::rpc::InvalidParamsType::Required,
                message: e.to_string(),
              },
            })
          })?;

          let modify_member_game_setting =
            repositories::member_game_setting::models::ModifyMemberGameSetting {
              can_bet_casino: a.can_bet_casino,
              can_bet_slot: a.can_bet_slot,
              can_bet_powerball: a.can_bet_powerball,
              can_bet_powerladder: a.can_bet_powerladder,
              can_bet_eos: a.can_bet_eos,
              can_bet_bogglepowerball: a.can_bet_bogglepowerball,
            };
          self
            .member_game_setting_repository
            .update(&conn, id, &modify_member_game_setting)
            .map_err(|e| {
              bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                code: bpr::protobuf::rpc::Error::SERVER_00,
                message: format!("server {}", e),
                data: None,
              })
            })?;
        }

        if let Some(a) = request.settlement_setting {
          let id = uuid::Uuid::from_str(a.id.as_str()).map_err(|e| {
            bcr::error::rpc::Error::InvalidParams(bcr::error::rpc::InvalidParams {
              message: "invalid settlement_setting.id param".to_string(),
              detail: bcr::error::rpc::InvalidParamsDetail {
                location: "request".to_string(),
                param: "settlement_setting.id".to_string(),
                value: a.id.clone(),
                error_type: bcr::error::rpc::InvalidParamsType::Required,
                message: e.to_string(),
              },
            })
          })?;

          let modify_member_settlement_setting =
            repositories::member_settlement_setting::models::ModifyMemberSettlementSetting {
              can_exchange: a.can_exchange,
              can_first_charge_comp: a.can_first_charge_comp,
              can_per_charge_comp: a.can_per_charge_comp,
              manual_payment_type_for_partner:
                repositories::member_settlement_setting::schema::ManualPaymentType::from(
                  a.manual_payment_type_for_partner,
                ),
              settlement_type:
                repositories::member_settlement_setting::schema::SettlementType::from(
                  a.settlement_type,
                ),
              rate_casino: a.rate_casino,
              rate_casino_loosing: a.rate_casino_loosing,
              rate_casino_bacara: a.rate_casino_bacara,
              rate_casino_roulette: a.rate_casino_roulette,
              rate_casino_dragon_tiger: a.rate_casino_dragon_tiger,
              rate_slot: a.rate_slot,
              rate_slot_loosing: a.rate_slot_loosing,
              rate_powerball_single: a.rate_powerball_single,
              rate_powerball_combo: a.rate_powerball_combo,
              rate_powerladder_single: a.rate_powerladder_single,
              rate_powerladder_combo: a.rate_powerladder_combo,
              rate_eos_single: a.rate_eos_single,
              rate_eos_combo: a.rate_eos_combo,
              rate_bogglepowerball_single: a.rate_bogglepowerball_single,
              rate_bogglepowerball_combo: a.rate_bogglepowerball_combo,
            };
          self
            .member_settlement_setting_repository
            .update(&conn, id, &modify_member_settlement_setting)
            .map_err(|e| {
              bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                code: bpr::protobuf::rpc::Error::SERVER_00,
                message: format!("server {}", e),
                data: None,
              })
            })?;
        }

        let m = self.member_composition.select(&conn, id).map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        message
          .respond(
            bpr::ss::member::UpdateMemberResponse {
              error: None,
              result: Some(bpr::ss::member::update_member_response::Result {
                member: m.map(|d| bpr::models::member::MemberModel::from(&d)),
              }),
            }
            .encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::ss::member::UpdateMemberResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn update_member_for_password(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::ss::member::SUBJECT_UPDATE_MEMBER_FOR_PASSWORD,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let req = bpr::ss::member::UpdateMemberForPasswordRequest::decode(message.data.as_slice())
          .map_err(|e| {
            bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
              message: format!("invalid request: {}", e),
            })
          })?;

        let client = match req.client {
          Some(c) => c,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid client information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "client".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };
        let request = match req.request {
          Some(r) => r,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid request information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "request".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };

        let id = uuid::Uuid::from_str(request.id.as_str()).map_err(|e| {
          bcr::error::rpc::Error::InvalidParams(bcr::error::rpc::InvalidParams {
            message: "invalid id param".to_string(),
            detail: bcr::error::rpc::InvalidParamsDetail {
              location: "request".to_string(),
              param: "id".to_string(),
              value: request.id.clone(),
              error_type: bcr::error::rpc::InvalidParamsType::Required,
              message: e.to_string(),
            },
          })
        })?;

        let conn = self.pool.get().map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        let password_hash = match argon2::hash_encoded(
          request.password.as_bytes(),
          self.password_salt.as_bytes(),
          &self.argon2_config,
        ) {
          Ok(p) => p,
          Err(e) => {
            return Err(bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            }));
          }
        };

        let modify_member_for_password = repositories::member::models::ModifyMemberForPassword {
          password: password_hash,
        };

        let _affected = self
          .member_repository
          .update_password(&conn, id, &modify_member_for_password)
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        let m = self.member_composition.select(&conn, id).map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        message
          .respond(
            bpr::ss::member::UpdateMemberForPasswordResponse {
              error: None,
              result: Some(bpr::ss::member::update_member_for_password_response::Result {}),
            }
            .encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::ss::member::UpdateMemberForPasswordResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn update_member_for_state(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::ss::member::SUBJECT_UPDATE_MEMBER_FOR_STATE,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let req = bpr::ss::member::UpdateMemberForStateRequest::decode(message.data.as_slice())
          .map_err(|e| {
            bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
              message: format!("invalid request: {}", e),
            })
          })?;

        let client = match req.client {
          Some(c) => c,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid client information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "client".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };
        let request = match req.request {
          Some(r) => r,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid request information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "request".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };

        let id = uuid::Uuid::from_str(request.id.as_str()).map_err(|e| {
          bcr::error::rpc::Error::InvalidParams(bcr::error::rpc::InvalidParams {
            message: "invalid id param".to_string(),
            detail: bcr::error::rpc::InvalidParamsDetail {
              location: "request".to_string(),
              param: "id".to_string(),
              value: request.id.clone(),
              error_type: bcr::error::rpc::InvalidParamsType::Required,
              message: e.to_string(),
            },
          })
        })?;

        let conn = self.pool.get().map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        let modify_member_for_state = repositories::member::models::ModifyMemberForState {
          state: repositories::member::schema::MemberState::from(request.state),
        };

        let _affected = self
          .member_repository
          .update_state(&conn, id, &modify_member_for_state)
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        let m = self.member_composition.select(&conn, id).map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        message
          .respond(
            bpr::ss::member::UpdateMemberForStateResponse {
              error: None,
              result: Some(bpr::ss::member::update_member_for_state_response::Result {
                member: m.map(|d| bpr::models::member::MemberModel::from(&d)),
              }),
            }
            .encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::ss::member::UpdateMemberForStateResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn delete_member(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::ss::member::SUBJECT_DELETE_MEMBER,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let req =
          bpr::ss::member::DeleteMemberRequest::decode(message.data.as_slice()).map_err(|e| {
            bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
              message: format!("invalid request: {}", e),
            })
          })?;

        let client = match req.client {
          Some(c) => c,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid client information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "client".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };
        let request = match req.request {
          Some(r) => r,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid request information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "request".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };
        let id = uuid::Uuid::from_str(request.id.as_str()).map_err(|e| {
          bcr::error::rpc::Error::InvalidParams(bcr::error::rpc::InvalidParams {
            message: "invalid id param".to_string(),
            detail: bcr::error::rpc::InvalidParamsDetail {
              location: "request".to_string(),
              param: "id".to_string(),
              value: request.id.clone(),
              error_type: bcr::error::rpc::InvalidParamsType::Required,
              message: e.to_string(),
            },
          })
        })?;

        let conn = self.pool.get().map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        let deleted_at = chrono::Utc::now().timestamp();

        let modify_member = repositories::member::models::ModifyMember4DeletedAt {
          deleted_at: Some(deleted_at),
        };

        let m = self
          .member_repository
          .update_deleted_at(&conn, id, &modify_member)
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        message
          .respond(
            bpr::ss::member::DeleteMemberResponse {
              error: None,
              result: Some(bpr::ss::member::delete_member_response::Result {}),
            }
            .encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::ss::member::DeleteMemberResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }
}
